<template>
    <div class="player-wrapper">
        <div class="player-window-wrapper">
            <div
                v-if="wrapperName == 'h5-play-wrap'"
                class="h5-play-wrap"
            ></div>
            <div
                v-else
                :class="wrapperName"
                style="
                    width: 100%;
                    height: 100%;
                    background-color: #000;
                    border: 1px solid #333;
                    box-sizing: border-box;
                    position: relative;
                    overflow: hidden;
                "
            ></div>
        </div>
        <div class="tool-bar-wraper flex-end" v-if="showFull || showWdn">
            <el-popover
                placement="top"
                width="200"
                trigger="click"
                popper-class="window-type"
            >
                <div class="flex-center">
                    <i
                        class="btn-icon iconfont icon-sfz-fp1x1"
                        title="1x1"
                        @click="onChangeWdnNum('1')"
                    ></i>
                    <i
                        class="btn-icon iconfont icon-sfz-fp2x2"
                        title="2x2"
                        @click="onChangeWdnNum('2')"
                    ></i>
                    <i
                        class="btn-icon iconfont icon-sfz-fp3x3"
                        title="3x3"
                        @click="onChangeWdnNum('3')"
                    ></i>
                    <i
                        class="btn-icon iconfont icon-sfz-fp4x4"
                        title="4x4"
                        @click="onChangeWdnNum('4')"
                    ></i>
                </div>
                <i
                    slot="reference"
                    class="btn-icon el-icon-s-grid"
                    v-if="showWdn"
                ></i>
            </el-popover>

            <i
                v-if="showFull"
                class="btn-icon el-icon-rank"
                @click="onSetFull"
            ></i>
        </div>
    </div>
</template>

<script>
/**
 * @description 自定义选择器
 * @param {string} str dom元素
 */
function $(str) {
    if (str.charAt(0) == '#') {
        return document.getElementById(str.substring(1))
    } else if (str.charAt(0) == '.') {
        return document.getElementsByClassName(str.substring(1))
    } else {
        return document.getElementsByTagName(str)
    }
}
export default {
    name: 'videoPlayerDH',
    props: {
        wrapperName: {
            type: String,
            default: 'h5-play-wrap',
        },
        showWdn: {
            type: Boolean,
            default: true,
        },
        showFull: {
            type: Boolean,
            default: true,
        },
    },
    data() {
        return {
            curDeviceIndex: 0,
            deviceObject: {
                loginState: [],
                session: [],
            },
            $loginState: '',
            $stream: '',
            $volume: '',
            $canvas: '', //canvas播放视频DOM
            $video: '', //video播放视频DOM
            $canvas_ivs: '', //canvas绘图DOM
            $video_wrap: '', //视频外层Wrap
            $videoLoading: '', //“加载中...”提示
            player: null,
            playerInstance: [], //播放|回放实例数组
            channel: 0, //当前通道
            curEnlargeWnd: 0,
            onlineChannel: [], //当前成功拉流的视频通道数组
            WndIndex: 0, //宫格窗口Id
            ivsInstance: [], //canvas绘图实例数组
            curWndType: 1, //窗口分割类型
        }
    },
    created() {},
    mounted() {
        this.init()
    },
    beforeDestroy() {
        this.stopAll()
    },
    methods: {
        stopAll() {
            this.playerInstance.forEach((item) => {
                if (item) {
                    item.close()
                    item = null
                }
            })
        },
        play(videoData) {
            let {ip, cameraCode, dyAccount, dyPassword} = videoData
            if (this.playerInstance[this.WndIndex]) {
                this.playerInstance[this.WndIndex].close()
                this.playerInstance[this.WndIndex] = null
                console.log('停止')
            }
            let channel = cameraCode ? cameraCode : 1
            if (!dyAccount || !dyPassword) {
                this.$message.warning('摄像机数据获取失败！')
            }
            var options = {
                wsURL: `ws://${ip}/rtspoverwebsocket`,
                rtspURL: `rtsp://${ip}/cam/realmonitor?channel=${channel}&subtype=0&proto=Private3`,
                username: dyAccount,
                password: dyPassword,
            }
            this.$videoLoading.style.display = ''
            this.$videoLoading.innerText = '加载中...'
            let player = new PlayerControl(options)
            this.playerInstance[this.WndIndex] = player
            player.on('WorkerReady', () => {
                player.connect()
                this.$videoLoading.style.display = ''
                this.$videoLoading.innerText = '连接中...'
            })
            player.on('PlayStart', () => {
                this.$videoLoading.style.display = 'none'
                this.$videoLoading.innerText = ''
            })
            player.on('Error', (rs) => {
                console.log(rs, rs.description, 'error')
                let message = '打开失败！'
                if (rs.description) message = rs.description
                if (rs.errorCode == 101) message = ''
                this.$videoLoading.style.display = ''
                this.$videoLoading.innerText = message
            })
            player.init(this.$canvas, this.$video)
        },
        stop() {
            this.playerInstance[this.WndIndex].close()
        },
        /**
         * @description 初始化
         */
        init() {
            let initWidth = '50%'
            if (!this.showWdn) {
                initWidth = '100%'
            }
            let videoStr = ''
            for (var i = 0; i < 16; i++) {
                videoStr +=
                    '<div wnd-index="' +
                    i +
                    '" style="float: left; background-color: #000; position: relative; width: calc(' +
                    initWidth +
                    ' - 2px); height: calc(' +
                    initWidth +
                    ' - 2px);overflow:hidden;border:1px solid rgb(125,125,125)"><canvas id="h5_canvas_' +
                    i +
                    '_' +
                    this.wrapperName +
                    '" style="width:100%;height:100%;"></canvas><p id="h5_loading_' +
                    i +
                    '_' +
                    this.wrapperName +
                    '" class="video_loading"  style="display:none; position: absolute; top: 50%; left: 50%; color: #fff; transform:translate(-50%,-50%)">加载中...</p><video id="h5_video_' +
                    i +
                    '_' +
                    this.wrapperName +
                    '" style="display: none;width:100%;height:100%;position:absolute;top:0;left:0"></video><canvas id="h5_ivs_' +
                    i +
                    '_' +
                    this.wrapperName +
                    '" style="position: absolute;left: 0;top: 0;width: 100%;height: 100%;" width="500" height="300"></canvas></div>'
            }
            document.querySelector(`.${this.wrapperName}`).innerHTML = videoStr
            document
                .querySelectorAll(`.${this.wrapperName} div`)
                .forEach((item, index) => {
                    item.addEventListener('click', (event) => {
                        let _wndIndex =
                            event.target.parentNode.getAttribute('wnd-index') -
                            0
                        if (
                            !(
                                this.playerInstance[_wndIndex] &&
                                this.playerInstance[_wndIndex].isPlayback
                            )
                        ) {
                            this.channel =
                                event.target.parentNode.getAttribute(
                                    'channel'
                                ) - 0
                            document
                                .querySelectorAll('#h5_channel_list li')
                                .forEach((item) => {
                                    item.classList.remove('fn-fontRed')
                                })
                        }
                        document
                            .querySelectorAll(`.${this.wrapperName} div`)
                            .forEach((item, index) => {
                                if (index === _wndIndex && this.showWdn) {
                                    item.style.borderColor = 'rgb(255, 204, 0)'
                                    if (
                                        !(
                                            this.playerInstance[_wndIndex] &&
                                            this.playerInstance[_wndIndex]
                                                .isPlayback
                                        )
                                    ) {
                                        if (
                                            this.onlineChannel.indexOf(
                                                this.channel
                                            ) > -1
                                        ) {
                                            document
                                                .querySelector(
                                                    '#h5_channel_list li[channel="' +
                                                        this.channel +
                                                        '"]'
                                                )
                                                .classList.add('fn-fontRed')
                                        }
                                    }
                                    this.WndIndex = _wndIndex
                                    this.setVideoDom()
                                } else {
                                    item.style.borderColor =
                                        'rgb(125, 125, 125)'
                                }
                            })
                    })
                })
            this.$video_wrap = document.querySelector(`.${this.wrapperName}`)
            this.setVideoDom()
            ;[
                'fullscreenchange',
                'webkitfullscreenchange',
                'mozfullscreenchange',
                'msfullscreenchange',
            ].forEach((item, index) => {
                document.addEventListener(item, this.fullscreenchange, false)
            })
        },
        /**
         * @description 切换宫格时重新设置当前视频dom
         */
        setVideoDom() {
            this.$canvas = $(
                '#h5_canvas_' + this.WndIndex + '_' + this.wrapperName
            )
            this.$video = $(
                '#h5_video_' + this.WndIndex + '_' + this.wrapperName
            )
            this.$canvas_ivs = $(
                '#h5_ivs_' + this.WndIndex + '_' + this.wrapperName
            )
            this.$videoLoading = $(
                '#h5_loading_' + this.WndIndex + '_' + this.wrapperName
            )
        },
        /**
         * @description 切换窗口分割
         */
        onChangeWdnNum(val) {
            let ivsDom = document.querySelectorAll('[id^=h5_ivs_]')
            let divDom = document.querySelectorAll(`.${this.wrapperName} div`)
            this.curWndType = val
            if (val === '1') {
                divDom.forEach((item) => {
                    item.style.width = 'calc(100% - 2px)'
                    item.style.height = 'calc(100% - 2px)'
                    item.style.borderColor = '#000'
                })
            } else if (val === '2') {
                divDom.forEach((item, index) => {
                    item.style.width = 'calc(50% - 2px)'
                    item.style.height = 'calc(50% - 2px)'
                    if (index === 0) {
                        item.style.borderColor = ' rgb(255, 204, 0)'
                    } else {
                        item.style.borderColor = ' rgb(125, 125, 125)'
                    }
                })
            } else if (val === '3') {
                divDom.forEach((item, index) => {
                    item.style.height = 'calc(33.333% - 2px)'
                    item.style.width = 'calc(33.333% - 2px)'
                    if (index === 0) {
                        item.style.borderColor = ' rgb(255, 204, 0)'
                    } else {
                        item.style.borderColor = ' rgb(125, 125, 125)'
                    }
                })
            } else if (val === '4') {
                divDom.forEach((item, index) => {
                    item.style.width = 'calc(25% - 2px)'
                    item.style.height = 'calc(25% - 2px)'
                    if (index === 0) {
                        item.style.borderColor = 'rgb(255, 204, 0)'
                    } else {
                        item.style.borderColor = 'rgb(125, 125, 125)'
                    }
                })
            }
            ivsDom.forEach((item) => {
                item.setAttribute('width', `${item.parentNode.clientWidth}`)
                item.setAttribute('height', `${item.parentNode.clientHeight}`)
            })
            this.ivsInstance.forEach((item) => {
                item && item.resize()
            })
            document.querySelector('#h5_ivs_0').click()
        },
        /**
         * @description 设置全屏
         */
        onSetFull() {
            debugger
            if (this.getFull()) {
                this.exitfullScreen()
            } else {
                this.setfullScreen()
            }
        },
        /**
         * @description 获取全屏状态
         */
        getFull() {
            return (
                window.top.document.mozFullScreen ||
                window.top.document.webkitIsFullScreen ||
                window.top.document.msFullscreenElement
            )
        },
        /**
         * @description 全屏状态改变的回调事件
         */
        fullscreenchange() {
            if (this.getFull()) {
                return
            } else {
                this.exitfullScreen()
            }
        },
        /**
         * @description 设置全屏
         */
        setfullScreen() {
            let docElm = window.top.document.documentElement
            if (docElm.requestFullScreen) {
                docElm.requestFullScreen()
            } else if (docElm.mozRequestFullScreen) {
                docElm.mozRequestFullScreen()
            } else if (docElm.webkitRequestFullScreen) {
                docElm.webkitRequestFullScreen()
            } else if (docElm.msRequestFullscreen) {
                docElm.msRequestFullscreen()
            }
            this.handleFullscreen(true)
        },
        /**
         * @description 退出全屏
         */
        exitfullScreen() {
            let docElm = window.top.document.documentElement
            if (docElm.exitFullscreen) {
                docElm.exitFullscreen()
            } else if (docElm.mozCancelFullScreen) {
                docElm.mozCancelFullScreen()
            } else if (docElm.webkitCancelFullScreen) {
                docElm.webkitCancelFullScreen()
            } else if (docElm.msExitFullscreen) {
                docElm.msExitFullscreen()
            }
            this.handleFullscreen(false)
        },
        /**
         * @description 处理全屏开关时的窗口大小
         * @param {boolean} bool 是否要全屏
         */
        handleFullscreen(bool) {
            if (bool) {
                let wrap = {
                    position: 'fixed',
                    left: 0,
                    top: 0,
                    width: window.screen.width + 'px',
                    height: window.screen.height + 'px',
                    overflow: 'visible',
                    zIndex: 1,
                }
                this.setStyle(this.$video_wrap, wrap)
            } else {
                let wrap = {
                    position: 'relative',
                    overflow: 'hidden',
                    width: '100%',
                    height: '100%',
                }
                this.setStyle(this.$video_wrap, wrap)
            }
        },

        /**
         * @description 设置样式
         * @param {object} obj dom元素
         * @param {*} json css样式
         */
        setStyle(obj, json) {
            for (let i in json) {
                obj.style[i] = json[i]
            }
        },
        /**
         * @description 登录
         */
        onLogin(ip) {
            this.playerInstance.forEach((item) => {
                if (item) {
                    item.close()
                    item = null
                }
            })
            // $('#h5_channel_list').innerHTML = ''
            // $('#h5_playback_channel').options.length = 0
            this.onlineChannel = []
            let port = '80'
            let name = 'admin'
            let pswd = 'CQ@hik12345.'
            let target = ip + ':' + port
            setIP(target)
            RPC.login(name, pswd, false)
                .then((res) => {
                    setCookie('DWebClientSessionID', '', -1)
                    setCookie('DhWebClientSessionID', '', -1)
                    /**
                     * RPC.keepAlive 保活
                     */
                    RPC.keepAlive(
                        300,
                        60000,
                        _getSession(),
                        target,
                        this.WndIndex
                    )
                    this.afterLogin()
                    this.onRealPreview(ip, port, name, pswd) //登录后拉流
                })
                .catch((err) => {
                    console.log(err)
                    loginError(err)
                    this.$videoLoading.style.display = ''
                    this.$videoLoading.innerText = '打开失败！'
                })
        },
        /**
         * @description 登录之后调用，获取设备能力，获取通道、码流
         */
        afterLogin() {
            this.deviceObject.loginState[this.curDeviceIndex] = true
            this.deviceObject.session[this.curDeviceIndex] = _getSession()
            this.$stream.options.length = 0
            $('#h5_playback_channel').options.length = 0
            $('#h5_channel').options.length = 0
            /**
             * RPC.MagicBox.getProductDefinition 获取产品定义
             * @param {string}  'MaxExtraStream' 定义名称
             * @returns {Promise}
             */
            RPC.MagicBox.getProductDefinition('MaxExtraStream').then(
                (params) => {
                    let maxExtra = params.definition
                    this.$stream.options.add(new Option('主码流', 0))
                    if (maxExtra > 1) {
                        for (let i = 1; i <= maxExtra; i++) {
                            this.$stream.options.add(
                                new Option('辅码流' + i, i)
                            )
                        }
                    } else {
                        this.$stream.options.add(new Option('辅码流', 1))
                    }
                }
            )
            let curDate = new Date()
            let dateString = curDate.toLocaleDateString()
            let dateSplit = dateString.split('/')
            let month = dateSplit[1] - 0
            if (month < 10) {
                dateSplit[1] = '0' + month
            }
            let day = dateSplit[2] - 0
            if (day < 10) {
                dateSplit[2] = '0' + day
            }
            let date = dateSplit.join('-')
            $('#h5_startTime').value = date + 'T' + '00:00'
            $('#h5_endTime').value = date + 'T' + '23:59:59'
        },
        onRealPreview(ip, port, name, pswd) {
            if (this.playerInstance[this.WndIndex]) {
                this.playerInstance[this.WndIndex].stop()
                this.playerInstance[this.WndIndex].close()
                this.playerInstance[this.WndIndex] = null
                let dom = this.$canvas
                if (dom.style.display === 'none') {
                    dom = this.$video
                }
                dom.style.display = 'none'
            }
            var player = null
            let curChannel = channel + 1 //无插件通道号从1开始
            let stream = this.$stream.value - 0
            let options = {
                wsURL: 'ws://' + ip + ':' + port + '/rtspoverwebsocket',
                rtspURL:
                    'rtsp://' +
                    ip +
                    ':' +
                    port +
                    '/cam/realmonitor?channel=' +
                    curChannel +
                    '&subtype=' +
                    stream +
                    '&proto=Private3',
                username: name,
                password: pswd,
                lessRateCanvas: true,
            }
            player = new PlayerControl(options)
            player.on('MSEResolutionChanged', (e) => {
                console.log(e)
            })
            player.on('PlayStart', (e) => {
                console.log(e)
                this.$canvas.parentNode.setAttribute(
                    'deviceindex',
                    this.curDeviceIndex
                )
                this.$videoLoading.style.display = 'none'
                // let curWndType =
                //     _doc.querySelector('[sel-for=onChangeWdnNum]').value - 0
                // let _newDom = _doc.querySelector(
                //     '.h5-menu-list li:nth-child(' + (curDeviceIndex + 2) + ')'
                // )
                // let hasLoginNum = getLoginState()
                // if (
                //     curWndType !== 1 &&
                //     _newDom != null &&
                //     isStartAll &&
                //     hasLoginNum < curWndType * curWndType
                // ) {
                //     clickNextWnd()
                //     _newDom.click()
                // } else {
                //     isStartAll = false
                // }
            })
            player.on('DecodeStart', (e) => {
                console.log(e)
                if (e.decodeMode === 'video') {
                    this.$canvas.style.display = 'none'
                    this.$video.style.display = ''
                } else {
                    this.$video.style.display = 'none'
                    this.$canvas.style.display = ''
                }
                canvasSon = new PluginCanvasES6()

                canvasSon.init(this.$canvas_ivs, (data) => {
                    this.rebackActivateLocalEnlarging(data)
                })
                canvasSon.addChangeShapeEvent()
                this.playerInstance[WndIndex] = player
                this.ivsInstance[WndIndex] = canvasSon
            })
            player.on('GetFrameRate', function (e) {
                console.log('GetFrameRate: ' + e)
            })
            player.on('FrameTypeChange', function (e) {
                console.log('FrameTypeChange: ' + e)
            })
            player.on('Error', function (e) {
                //console.log('Error: ' + JSON.stringify(e))
            })
            player.on('WorkerReady', function () {
                player.connect()
            })
            player.init(this.$canvas, this.$video)
            this.$videoLoading.style.display = ''
            this.$videoLoading.innerText = '加载中...'
        },
        /**
         * @description 设置登录状态
         * @param {boolean} bool 设备是否已经登录
         */
        setLoginState(bool) {
            isLogin = bool
        },
        /**
         * @description 绘制电子放大后的回调函数
         * @param {object} data 矩形框的坐标信息
         */
        rebackActivateLocalEnlarging(data) {
            if (this.curEnlargeWnd != this.WndIndex) return
            let pos = data.data
            let newData
            if (pos[0][0] === pos[1][0]) {
                // return false;
            } else {
                newData = {
                    left: pos[0][0],
                    top: pos[0][1],
                    right: pos[1][0],
                    bottom: pos[1][1],
                }
            }
            let dom = this.$canvas
            if (dom.style.display === 'none') {
                dom = this.$video
            }
            // 倒着画
            if (newData.right < newData.left) {
                let tmp = newData.left
                newData.left = newData.right
                newData.right = tmp
            }

            if (newData.bottom < newData.top) {
                let tmp = newData.top
                newData.top = newData.bottom
                newData.bottom = tmp
            }

            let scaleW =
                this.$video_wrap.childNodes[this.WndIndex].clientWidth / 8191
            let scaleH =
                this.$video_wrap.childNodes[this.WndIndex].clientHeight / 8191

            let result = zoomArea(
                newData.left * scaleW,
                newData.top * scaleH,
                newData.right * scaleW,
                newData.bottom * scaleH,
                this.$video_wrap.childNodes[this.WndIndex].clientWidth,
                this.$video_wrap.childNodes[this.WndIndex].clientHeight
            )
            dom.style.width = result.width + 'px'
            dom.style.height = result.height + 'px'
            dom.style.left = result.left + 'px'
            dom.style.top = result.top + 'px'
            dom.style.position = 'absolute'
            this.ivsInstance[this.WndIndex].removeShapeDrawEvent()
        },
    },
}
</script>

<style lang="scss" scoped>
.player-wrapper{
    width: 100%;
    height: 100%;
    box-sizing: border-box;
}
.player-window-wrapper{
    width: 100%;
    height: calc(100% - 40px);
    position: relative;
    overflow: hidden;
    box-sizing: border-box;
}
.h5-play-wrap {
    width: 100%;
    height: 100%;
    background-color: #000;
    border: 1px solid #333;
    box-sizing: border-box;
    position: relative;
    overflow: hidden;
}
.tool-bar-wraper{
    position: relative;
    height: 40px;
    width: 100%;
    box-sizing: border-box;
    background-color: rgba(0,0,0,0.6);
    z-index: 0;
}
.btn-icon{
    font-size: 20px;
    color: #cecece;
    margin: 10px 5px;
    cursor: pointer;
}
.btn-icon:hover{
    color: #e0e0e0;
}
.el-icon-rank{
    font-size: 22px;
    transform: rotate(45deg);
}
.video_loading{
    position: absolute;
    top: 0;
    left: 0;
    color: #fff;
}
</style>
<style lang='scss'>
.window-type{
    background-color: #333;
    border-color:#333;
    min-width:120px !important;
    width: 120px!important;
}
</style>